function [s_locs,r_ms] = localize_heartsounds(signal,decompose_level,entropy_smooth,rms_refine_size,rms_refine_th,label_min_diff)
%LOCALIZE_HEARTSOUNDS Locates high probability heartsound events in time
%   Returned values:
%       - centroids of heartsounds
%       - rms envelope of the signal
%   Input parameters:
%       - signal
%       - level of wavelet decomposition for detection
%       - entropy smoothing window size in ms
%       - rms envelope based refinement search window
%       - minimal rms value for detection
%       - minimal time delay between elements
    
    % normalize and denoise
    signal=signal/abs(max(signal));
    den = wdenoise(signal,8,'Wavelet','coif5','DenoisingMethod','SURE','NoiseEstimate','LevelDependent');
    
    % decompose for detection
    [c,l] = wavedec(den,decompose_level+2,'db6');
    det = detcoef(c,l,'db6');

    % movmean smoothed Shannon entropy
    win_size = pow2(decompose_level)/entropy_smooth;
    se = smooth_entropy(det{decompose_level},win_size);
    
    % instantaneous phase
    inst_ph = angle(hilbert(se));

    % resample inst. phase
    re_se = resample(inst_ph,pow2(decompose_level),1);

    % positive slope zero crossing
    pos_zc = find(diff(sign(re_se))>0);
    zc_locs = pos_zc-entropy_smooth;
    zc_locs(zc_locs<0)=[];

    s_locs= [];
    r_ms = envelope(den,50,'rms');
    for k=1:length(zc_locs)

        % window rms around locations
        l = zc_locs(k);
        st = l-rms_refine_size;
        en = l+rms_refine_size;
        if st <= 0
            st=1;
        end
        if en > length(r_ms)
            en=length(r_ms)-1;
        end
        s = r_ms(st:en);
        
        % threshold rms and refine location to centroid
        if sum(s) > rms_refine_th
            ref = cumsum(s)-(sum(s)/2);
            [~,r] = min(abs(ref));
    
            s_locs = [s_locs, l-rms_refine_size+r];
    
        end
    end
    
    % remove dense labels
    s_locs(diff(s_locs)<label_min_diff) = [];
end


